@@ -120,13 +120,14 @@ instance Functor (STB bs gs) where
   fmap = liftM
 
 instance Applicative (STB bs gs) where
-  pure  = return
+  pure  = yield
   (<*>) = ap
+  (*>)  = (+>)
 
 instance Monad (STB bs gs) where
-  return = yield
+  return = pure
   (>>=)  = (+>=)
-  (>>)   = (+>)
+  (>>)   = (*>)
 
 -- the monad's unit
 --
